@@ -3,6 +3,7 @@ Tools for reading a nexus file
 """
 import re
 import os
+import warnings
 
 try:
     import io
@@ -18,6 +19,8 @@ NCHAR_PATTERN = re.compile(r"""nchar=(\d+)""", re.IGNORECASE)
 COMMENT_PATTERN = re.compile(r"""(\[.*?\])""")
 WHITESPACE_PATTERN = re.compile(r"""\s+""")
 QUOTED_PATTERN = re.compile(r"""^["'](.*)["']$""")
+MESQUITE_TITLE_PATTERN = re.compile(r"""^TITLE\s+(.*);$""", re.IGNORECASE)
+MESQUITE_LINK_PATTERN = re.compile(r"""^LINK\s+(.*?)\s+=\s+(.*);$""", re.IGNORECASE)
 
 
 class NexusFormatException(Exception):
@@ -91,6 +94,7 @@ class TaxaHandler(GenericHandler):
     
     def __init__(self):
         self.taxa = []
+        self.attributes = []
         self.ntaxa = 0
         super(TaxaHandler, self).__init__()
         
@@ -118,7 +122,13 @@ class TaxaHandler(GenericHandler):
             elif self.is_taxlabel_block.match(line):
                 in_taxlabel_block = True
             elif in_taxlabel_block:
-                self.taxa.append(line.strip(";"))
+                for taxon in line.strip(';').split():
+                    self.taxa.append(taxon)
+            elif MESQUITE_TITLE_PATTERN.match(line):
+                self.attributes.append(line)
+            elif MESQUITE_LINK_PATTERN.match(line):
+                self.attributes.append(line)
+            
         assert self.ntaxa == len(self.taxa)
         
     def write(self):
@@ -127,7 +137,13 @@ class TaxaHandler(GenericHandler):
         
         :return: String
         """
-        out = ['begin taxa;', 'dimensions ntax=%d' % self.ntaxa, 'taxlabels']
+        out = ['begin taxa;']
+        # handle any attributes
+        for att in self.attributes:
+            out.append("\t%s" % att)
+        out.append('\tdimensions ntax=%d;' % self.ntaxa)
+        out.append('\ttaxlabels')
+        # taxa labels
         for idx, taxon in enumerate(self.taxa, 1):
             out.append("\t[%d] '%s'" % (idx, taxon))
         out.append(';')
@@ -143,10 +159,20 @@ class TreeHandler(GenericHandler):
     """Handler for `trees` blocks"""
     is_tree = re.compile(r"""tree .*=.*;""", re.IGNORECASE)
     
+    translate_regex = re.compile(r"""
+        ([,(])              # boundary
+        ([A-Z0-9_\-\.]+)    # taxa-id
+        :?                # optional colon
+        (\[.+?\])?          # minimally match an optional comment chunk
+        (\d+\.\d+)?         # optional branchlengths
+        (?=[),])?           # end bounday - n.b. lookahead stops the next pattern being consumed
+    """, re.IGNORECASE + re.VERBOSE + re.DOTALL)
+    
     def __init__(self):
         self.was_translated = False # does the treefile have a translate block?
         self._been_detranslated = False # has detranslate been called?
         self.translators = {}
+        self.attributes = []
         self.taxa = []
         self.trees = []
         super(TreeHandler, self).__init__()
@@ -178,6 +204,11 @@ class TreeHandler(GenericHandler):
             if translate_start.match(line):
                 lost_in_translation = True
                 self.was_translated = True
+            # MESQUITE attributes
+            elif MESQUITE_TITLE_PATTERN.match(line):
+                self.attributes.append(line)
+            elif MESQUITE_LINK_PATTERN.match(line):
+                self.attributes.append(line)
                 
             # if we're in a translate block
             elif lost_in_translation:
@@ -210,6 +241,21 @@ class TreeHandler(GenericHandler):
         for idx, tree in enumerate(self.trees):
             self.trees[idx] = self._detranslate_tree(tree, self.translators)
         self._been_detranslated = True
+    
+    def _findall_chunks(self, tree):
+        """Helper function to find groups used by detranslate."""
+        matches = []
+        index = 0
+        while True:
+            match = self.translate_regex.search(tree, index)
+            if not match:
+                break
+            m = dict(zip(['start', 'taxon', 'comment', 'branch'], match.groups()))
+            m['match'] = tree[match.start():match.end() + 1]
+            m['end'] = tree[match.end()]
+            matches.append(m)
+            index = match.end()
+        return matches
         
     def _detranslate_tree(self, tree, translatetable):
         """
@@ -224,36 +270,26 @@ class TreeHandler(GenericHandler):
         
         :return: String of detranslated tree
         """
-        # regex = re.compile(r"""
-        # [,\(]                   # boundary
-        #  (
-        #     (\d+)               # taxa-id
-        #     (\[.+?\])?          # minimally match an optional comment chunk
-        #     (:\d+\.\d+)?        # optional branchlengths
-        #  )
-        # [,\)]                   # end!
-        # """, re.IGNORECASE + re.VERBOSE + re.DOTALL)
-        if '[' in tree:
-            # beast mode comes first - it also has branchlengths
-            MODE = 'beast'
-            regex = re.compile(r"((\d+)[\[:])")
-        elif ':' in tree:
-            MODE = 'branchlengths'
-            regex = re.compile(r"((\d+):)")
-        else:
-            MODE = 'simple'
-            regex = re.compile(r"((\d+))")
-        
-        for found in regex.finditer(tree):
-            old, tax_id = found.groups()
-            if tax_id in translatetable:
-                taxon = translatetable[tax_id]
-                if MODE == 'branchlengths':
-                    tree = re.sub(r"\b(%s:)\b" % tax_id, "%s:" % taxon, tree)
-                elif MODE == 'beast':
-                    tree = re.sub(r"\b(%s)([\[:])" % tax_id, "%s:" % taxon, tree)
-                elif MODE == 'simple':
-                    tree = re.sub(r"\b(%s)\b" % tax_id, taxon, tree)
+        for found in self._findall_chunks(tree):
+            if found['taxon'] in translatetable:
+                taxon = translatetable[found['taxon']]
+                if found['comment'] and found['branch']:
+                    # comment and branch
+                    sub = "%s:%s%s" % (taxon, found['comment'], found['branch'])
+                elif found['comment']:
+                    # comment only
+                    sub = "%s%s" % (taxon, found['comment'])
+                elif found['branch']:
+                    # branch only
+                    sub = "%s:%s" % (taxon, found['branch'])
+                else: 
+                    # taxon only
+                    sub = taxon
+                sub = "%s%s%s" % (found['start'], sub, found['end'])
+                if found['match'] not in tree:
+                    raise ValueError("Expected match for %s not in tree" % found['match'])
+                tree = tree.replace(found['match'], sub)
+            
         return tree
         
     def write(self):
@@ -263,6 +299,8 @@ class TreeHandler(GenericHandler):
         :return: String
         """
         out = ['begin trees;']
+        for attr in self.attributes:
+            out.append("\t"+ attr)
         if self.was_translated and self._been_detranslated == False:
             out.append('\ttranslate')
             for index in sorted([int(k) for k in self.translators.keys()]):
@@ -283,12 +321,10 @@ class DataHandler(GenericHandler):
     _character_block_pattern = re.compile(r"""CHARSTATELABELS(.*?);""", re.IGNORECASE | re.DOTALL)
     
     def __init__(self):
-        self.taxa = []
-        self.ntaxa = 0
         self.symbols = set()
         self.characters = {}
         self.charlabels = {}
-        self.nchar = 0
+        self.attributes = []
         self.format = {}
         self.gaps = None
         self.missing = None
@@ -297,7 +333,22 @@ class DataHandler(GenericHandler):
 
     def __getitem__(self, index):
         return (self.taxa[index], self.matrix[self.taxa[index]])
+    
+    @property
+    def ntaxa(self):
+        """Number of Taxa"""
+        return len(self.matrix)
+        
+    @property
+    def nchar(self):
+        """Number of Characters"""
+        return len(self.matrix[self.matrix.keys()[0]])
 
+    @property
+    def taxa(self):
+        """Taxa list"""
+        return self.matrix.keys()
+    
     def parse_format_line(self, data):
         """
         Parses a format line, and returns a dictionary of tokens
@@ -378,6 +429,32 @@ class DataHandler(GenericHandler):
         if multistate:
             raise NexusFormatException("Data Matrix contains incomplete multistate values")
         return out
+    
+    def add_taxon(self, taxon, site_values = None):
+        """
+        Adds a `taxon` to the matrix with values from `site_values`
+        
+        :param taxon: taxa name
+        :type data: string
+        
+        :param site_values: site values
+        :type data: list
+        
+        :return: None
+        """
+        self.matrix[taxon] = self.matrix.get(taxon, [])
+        self.matrix[taxon].extend(site_values)
+        
+    def del_taxon(self, taxon):
+        """
+        Removes `taxon` from the data
+        
+        :param taxon: taxa name
+        :type data: string
+        
+        :return: None
+        """
+        del(self.matrix[taxon])
         
     def parse(self, data):
         """
@@ -394,19 +471,30 @@ class DataHandler(GenericHandler):
         self.format = self.parse_format_line("\n".join(data))
         data = self._parse_charstate_block(data)
         
+        _dimensions_found_taxa, _dimensions_found_chars = None, None
+        
         seen_matrix = False
         for line in data:
             lline = line.lower().strip()
             # Dimensions line
             if lline.startswith('dimensions '):
-                # try for nchar/ntax
+                # try for ntaxa
                 try:
-                    self.ntaxa = int(NTAX_PATTERN.findall(line)[0])
+                    _dimensions_found_taxa = int(NTAX_PATTERN.findall(line)[0])
                 except IndexError:
-                    self.ntaxa = None
-
-                self.nchar = int(NCHAR_PATTERN.findall(line)[0])
-
+                    pass
+                # and nchar
+                try:
+                    _dimensions_found_chars = int(NCHAR_PATTERN.findall(line)[0])
+                except IndexError:
+                    pass
+                
+            # handle MESQUITE attributes
+            elif MESQUITE_TITLE_PATTERN.match(line):
+                self.attributes.append(line)
+            elif MESQUITE_LINK_PATTERN.match(line):
+                self.attributes.append(line)
+            
             # handle format line
             elif lline.startswith('format'):
                 continue
@@ -425,25 +513,17 @@ class DataHandler(GenericHandler):
                 except ValueError:
                     continue
                 
-                taxon = taxon.strip()
-                taxon = QUOTED_PATTERN.sub('\\1', taxon)
-                sites = sites.strip()
-                
-                if taxon not in self.taxa:
-                    self.taxa.append(taxon)
-                    
-                self.matrix[taxon] = self.matrix.get(taxon, [])
-                self.matrix[taxon].extend(self._parse_sites(sites))
+                taxon = QUOTED_PATTERN.sub('\\1', taxon.strip())
+                self.add_taxon(taxon, self._parse_sites(sites.strip()))
                 
         self._load_characters()
         
-        # WARNING: what if ntaxa and nchar in format string does not give us the right answer?
-        # Should we raise an error instead?
-        if self.ntaxa != len(self.taxa):
-            self.ntaxa = len(self.taxa)
-        
-        if self.nchar != len(self.characters):
-            self.nchar = len(self.characters)
+        # Raise Warnings if ntaxa or nchar in format string does not give us the right answer
+        if _dimensions_found_taxa is not None and self.ntaxa != _dimensions_found_taxa:
+            warnings.warn("Expected %d taxa, got %d" % (self.ntaxa, _dimensions_found_taxa))
+            
+        if _dimensions_found_chars is not None and self.nchar != _dimensions_found_chars:
+            warnings.warn("Expected %d characters, got %d" % (self.nchar, _dimensions_found_chars))
     
     def _load_characters(self):
         """Loads characters into self.characters section"""
@@ -502,6 +582,8 @@ class DataHandler(GenericHandler):
 
         out = []
         out.append('begin data;')
+        for att in self.attributes:
+            out.append("\t%s" % att)
         out.append('\tdimensions ntax=%d nchar=%d;' % (self.ntaxa, self.nchar))
         out.append(_make_format_line(self))
         out.append("matrix")
@@ -556,10 +638,15 @@ class NexusReader(object):
         """
         self.filename = filename
         self.short_filename = os.path.split(filename)[1]
-        try:
-            handle = open(filename, 'rU')
-        except IOError:
+        
+        if os.path.isfile(filename) == False:
             raise IOError("Unable To Read File %s" % filename)
+        
+        if filename.endswith('.gz'):
+            import gzip
+            handle = gzip.open(filename, 'rb')
+        else:
+            handle = open(filename, 'rU')
         self._read(handle)
         handle.close()
     
