@@ -4,7 +4,7 @@ import unittest
 from nexus import NexusReader, NexusWriter, NexusFormatException
 from nexus.tools import find_constant_sites
 
-EXAMPLE_DIR = os.path.join(os.path.split(os.path.dirname(__file__))[0], '../../examples')
+EXAMPLE_DIR = os.path.join(os.path.dirname(__file__), '../../examples')
 
 
 class Test_FindConstantSites(unittest.TestCase):
