@@ -4,7 +4,7 @@ import unittest
 from nexus import NexusReader, NexusWriter, NexusFormatException
 from nexus.tools import new_nexus_without_sites
 
-EXAMPLE_DIR = os.path.join(os.path.split(os.path.dirname(__file__))[0], '../../examples')
+EXAMPLE_DIR = os.path.join(os.path.dirname(__file__), '../../examples')
 
 class Test_NewNexusWithoutSites(unittest.TestCase):
     """Tests for new_nexus_without_sites"""
