@@ -5,7 +5,7 @@ import unittest
 from nexus import NexusReader, NexusWriter, NexusFormatException
 from nexus.bin.nexus_treemanip import run_resample
 
-EXAMPLE_DIR = os.path.join(os.path.split(os.path.dirname(__file__))[0], '../../examples')
+EXAMPLE_DIR = os.path.join(os.path.dirname(__file__), '../../examples')
 
 class Test_ResampleTrees(unittest.TestCase):
     """Test nexus_treemanip.run_resample"""
